@@ -4,14 +4,16 @@ import re
 
 from ClickSQL.clickhouse.ClickHouse import ClickHouseTableNode
 from ClickSQL.errors import ClickHouseTableExistsError, ParameterTypeError
+import warnings
 
 
 class SQLBuilder(object):
 
     @staticmethod
     def _assemble_sample(sample=None) -> str:
+        warnings.warn('currently sampling is not supported!')
         sample_clause = '' if sample is None else f'SAMPLE {sample}'
-        return sample_clause
+        return ''
 
     @staticmethod
     def _assemble_array_join(array_join_list: (list, tuple, None) = None) -> str:
@@ -25,18 +27,22 @@ class SQLBuilder(object):
         if join_info_dict is None:
             join_clause = ''
         elif isinstance(join_info_dict, dict):
-            join_type = join_info_dict.get('type')
+            join_type = join_info_dict.get('type', None)
+            sql = join_info_dict.get('sql', None)
             if join_type is None:
                 raise ValueError('join_info_dict cannot locate join_type condition')
+            if sql is None:
+                raise ValueError('join_info_dict cannot locate sql clause')
+
             on_ = join_info_dict.get('ON')
             if on_ is None:
                 using_ = join_info_dict.get('USING')
                 if using_ is None:
                     raise ValueError('join_info_dict cannot locate ON or USING condition')
                 else:
-                    join_clause = f'{join_type} USING ({using_})'
+                    join_clause = f'{join_type} ({sql}) USING ({using_})'
             else:
-                join_clause = f'{join_type} ON {on_}'
+                join_clause = f'{join_type} ({sql}) ON {on_}'
         else:
             raise ValueError('join_info_dict must accept dict or None')
         return join_clause
@@ -127,21 +133,22 @@ class SQLBuilder(object):
         join = f"{array_join_clause} {join_clause}"
         where_conditions = f"{prewhere_clause} {where_clause} {group_by_clause} {having_clause} "
         order_limit = f"{order_by_clause} {limit_n_clause} {limit_clause}"
-        sql = f"{main_body} {join} {where_conditions} {order_limit}"
+        sql = f"{main_body} {join} {where_conditions} {order_limit} SETTINGS joined_subquery_requires_alias=0"
         return sql
 
     @classmethod
-    def selector(cls, db_table: str, cols: list,
-                 sample: (int, float, None) = None,
-                 array_join: (list, None) = None,
-                 join: (dict, None) = None,
-                 prewhere: (list, None) = None,
-                 where: (list, None) = None,
-                 having: (list, None) = None,
-                 group_by: (list, None) = None,
-                 order_by: (list, None) = None,
-                 limit_by: (dict, None) = None,
-                 limit: (int, None) = None) -> str:
+    def select(cls, db_table: str,
+               cols: (list, str, None) = None,
+               sample: (int, float, None) = None,
+               array_join: (list, None) = None,
+               join: (dict, None) = None,
+               prewhere: (list, None) = None,
+               where: (list, None) = None,
+               having: (list, None) = None,
+               group_by: (list, None) = None,
+               order_by: (list, None) = None,
+               limit_by: (dict, None) = None,
+               limit: (int, None) = None) -> str:
         """
 
         :param having: str ["r1 >1 and r2 <2"]
@@ -149,7 +156,7 @@ class SQLBuilder(object):
         :param cols: list [ r1,r2,r3 ]
         :param sample: str 0.1 or 1000
         :param array_join: list ['arrayA as a','arrayB as b']
-        :param join: dict {'type':'all left join','USING' : "r1,r2"}
+        :param join: dict {'type':'all left join','USING' : "r1,r2",'sql':'select * from test'}
         :param prewhere: str ["r1 >1" , "r2 <2"]
         :param where: str ["r1 >1.5" , "r2 <1.3"]
         :param group_by: list ['r1','r2']
@@ -158,8 +165,14 @@ class SQLBuilder(object):
         :param limit: int 100
         :return:  str
         """
-
-        SELECT_CLAUSE = ','.join(cols)
+        if isinstance(cols, str):
+            select_clause = cols
+        elif isinstance(cols, list):
+            select_clause = ','.join(cols)
+        elif cols is None:
+            select_clause = '*'
+        else:
+            raise ValueError('cols only accept str or list')
         SAMPLE_CLAUSE = cls._assemble_sample(sample=sample)
         ARRAY_JOIN_CLAUSE = cls._assemble_array_join(array_join_list=array_join)
         JOIN_CLAUSE = cls._assemble_join(join)
@@ -171,7 +184,7 @@ class SQLBuilder(object):
         LIMIT_N_CLAUSE = cls._assemble_limit_by(limit_by)
         LIMIT_CLAUSE = cls._assemble_limit(limit)
 
-        return cls.raw_create_select_sql(SELECT_CLAUSE, db_table, SAMPLE_CLAUSE, ARRAY_JOIN_CLAUSE, JOIN_CLAUSE,
+        return cls.raw_create_select_sql(select_clause, db_table, SAMPLE_CLAUSE, ARRAY_JOIN_CLAUSE, JOIN_CLAUSE,
                                          PREWHERE_CLAUSE, WHERE_CLAUSE, GROUP_BY_CLAUSE, HAVING_CLAUSE, ORDER_BY_CLAUSE,
                                          LIMIT_N_CLAUSE, LIMIT_CLAUSE)
 
@@ -640,7 +653,7 @@ class CreateTableUtils(CreateTableFromInfoUtils, CreateTableFromSQLUtils):
         return sql
 
 
-class TableEngineCreator(ClickHouseTableNode, CreateTableUtils):
+class TableEngineCreator(ClickHouseTableNode, CreateTableUtils, SQLBuilder):
     def __init__(self, conn_str: (str, None) = None, **kwarg):
         super(TableEngineCreator, self).__init__(conn_str=conn_str, **kwarg)
 
