@@ -5,7 +5,7 @@ from collections import namedtuple
 
 from ClickSQL.clickhouse.ClickHouseExt import ClickHouseTableNodeExt
 from ClickSQL.errors import ClickHouseTableNotExistsError
-
+from functools import wraps
 complex_sql_select_count = 4
 factor_parameters = ('dt', 'code', 'value', 'fid')
 ft_node = namedtuple('factortable', factor_parameters)
@@ -114,7 +114,7 @@ class BaseSingleFactorBaseNode(object):
 
     @property
     def __sql__(self):
-        return self.operator._get_sql(db_table=self.db_table, **self._kwargs)
+        return self.operator.get_sql(db_table=self.db_table, **self._kwargs)
 
     @property
     def __factor_id__(self):  # add iid function get factor table id
@@ -393,6 +393,17 @@ class BaseSingleFactorTableNode(BaseSingleFactorNode, MergeSQLUtils):
         self._execute = execute
         self._no_self_update = no_self_update
 
+    def explain(self, sql: str):
+        return self.operator(f'explain {sql}')
+
+    def drop_table(self, target: str):
+        if '.' not in target:
+            raise ValueError('drop table must tell correspond database')
+        self.operator(f'drop table if exists {target}')
+
+    def drop_db(self, target: str):
+        self.operator(f'drop database if exists {target}')
+
     def merge(self, seconds, using: (list, str, tuple), join_type='all full join',
               cols: (list, str, None) = None, ):
         if seconds.lower().startswith('select'):
@@ -488,7 +499,7 @@ class BaseSingleFactorTableNode(BaseSingleFactorNode, MergeSQLUtils):
 if __name__ == '__main__':
     v_st_dis_buy_info = BaseSingleFactorTableNode(
         'clickhouse://default:Imsn0wfree@47.104.186.157:8123/raw.v_st_dis_buy_info',
-        cols=['cust_no', 'product_id', 'money'],
+        cols=None,
         order_by_cols=['money asc'],
         money='money >= 1000000', limit='limit 10'
     )
